/*                                                                          
  Ϳ
   Sprites.h                 (VRAM y Pantallas Virtuales).                
                                                                          
   Librera grfica especfica para modo 13h (320x200x256).               
   Impresin de sprites y bitmaps con y sin mscaras.                     
                                                                          
   Para incluirla en cualquier programa C basta con usar la orden         
   #include "sprites.h"                   al principio del mismo.         
                                                                          
   Cdigo y texto:  [NoP] de Compiler SoftWare.                     
  1997 C.S.
*/

/*  Funciones implementadas:  */

void DrawSpriteC( int, int, int, int, char *, unsigned int );
void DrawSpriteC_Mask( int, int, int, int, char, char *, unsigned int );
void DrawSprite( int, int, int, int, char far *, unsigned int );
void DrawSprite_Mask( int, int, int, int, char, char far *, unsigned int );
void GetSprite( int, int, int, int, char far *, unsigned int );


/*                                                                          
  Ŀ
   DrawSpriteC( x, y, ancho, alto, bitmap, segmento );                    
                                                                          
   Dibuja en pantalla mediante bucles anidados el bitmap especificado     
   usando la funcin PutPixel() para ello, pudiendo hacerlo tanto en VRAM 
   como en pantallas virtuales.                                           
  
*/
void DrawSpriteC( int x, int y, int ancho, int alto, char *bitmap, 
		  unsigned int segmento )
{
int vx, vy;

 for( vy=0; vy<alto; vy++ )
  for( vx=0; vx<ancho; vx++ )
   PutPixel( x+vx, y+vy, bitmap[(vy*ancho)+vx], segmento );

}

/*                                                                          
  Ŀ
   DrawSpriteC_Mask( x, y, ancho, alto, bitmap, segmento );               
                                                                          
   Dibuja en pantalla mediante bucles anidados el bitmap especificado     
   usando la funcin PutPixel() para ello, pudiendo hacerlo tanto en VRAM 
   como en pantallas virtuales. Permite especificarle un color mscara o  
   color que no ha de ser dibujado, para evitar los pixels transparentes  
   del sprite y no alterar el fondo de la imagen sobre la que imprimimos. 
  
*/
void DrawSpriteC_Mask( int x, int y, int ancho, int alto, char colormask, 
		       char *bitmap, unsigned int segmento )
{
int vx, vy;
char color;

 for( vy=0; vy<alto; vy++ )
  for( vx=0; vx<ancho; vx++ )
  {
   color = bitmap[(vy*ancho)+vx];
   if( color != colormask )
    PutPixel( x+vx, y+vy, color, segmento );
  }
}


/*                                                                          
  Ŀ
   DrawSprite( x, y, ancho, alto, bitmap, segmento );                     
                                                                          
   Dibuja en pantalla mediante instrucciones assembler el bitmap especi-  
   ficado, pudiendo hacerlo tanto en VRAM como en pantallas virtuales.    
  
*/
void DrawSprite( int x, int y, int ancho, int alto, char far *bitmap, 
		 unsigned int segmento )
{
unsigned int offs;

 offs = (y*320)+x;

asm {
	push ds                               /* preservar el valor de DS */
	lds si, bitmap                        /* DS:SI = Bitmap[0] */
	mov es, [segmento]
	mov di, [offs]                        /* ES:DI = (x,y) */
	mov dx, [alto]                        /* DX = Contador bucle */
	mov bx, [ancho]                       /* para mayor velocidad */
     }
BucleSpriteY:
asm {
	mov cx, bx                            /* CX = Ancho */
	rep movsb                             /* movemos CX bytes */
	add di, 320                           /* siguiente lnea de pant. */
	sub di, bx
	dec dx                                /* hasta que DX = 0 */
	jnz BucleSpriteY

	pop ds                                /* recuperamos DS */
    }
}

/*                                                                          
  Ŀ
   DrawSprite_Mask( x, y, ancho, alto, colormask, bitmap, segmento );     
                                                                          
   Dibuja en pantalla mediante instrucciones assembler el bitmap especi-  
   ficado, pudiendo hacerlo tanto en VRAM como en pantallas virtuales.    
   Se le indica tambin el color mscara (color que no hay que dibujar).  
  
*/
void DrawSprite_Mask( int x, int y, int ancho, int alto, char colormask,
		 char far *bitmap, unsigned int segmento )
{
unsigned int offs;

 offs = (y*320)+x;

asm {
	push ds                               /* preservar el valor de DS */
	lds si, bitmap                        /* DS:SI = Bitmap[0] */
	mov es, [segmento]
	mov di, [offs]                        /* ES:DI = (x,y) */
	mov dx, [alto]                        /* DX = Contador bucle */
	mov bx, [ancho]                       /* para mayor velocidad */
	mov ah, [colormask]
    }

DrawLoopSprite:;
asm     mov cx, bx;                           /* cx = ancho */

LineLoopSprite:;
asm {
	mov al, byte ptr [si]                 /* cogemos un byte del sprte */
	cmp al, ah                            /* es el color mscara? */
	je NoPaintPixelSprite                 /* son iguales -> salta */
	mov es:[di], al
    }

NoPaintPixelSprite:;                          /* son iguales:no lo dibujes*/
asm {
	inc si                                /* siguiente punto del spr. */
	inc di                                /*   " "  punto de pantalla */
	dec cx 
	jnz LineLoopSprite                    /* hasta completar ScanLine */
	
	add di, 320                           /* fin de ScanLine: DI+=320 */
	sub di, bx                            /* DI -= Ancho */
	dec dx                                /* Hasta que DX = 0 */
	jnz DrawLoopSprite

	pop ds
    }

}

/*                                                                          
  Ŀ
   GetSprite( x, y, ancho, alto, bitmap, segmento );                      
                                                                          
   Lee el sprite o bloque desde (x,y) hasta (x+ancho,y+alto) y lo almacena
   en el array que se le especifica, que debe tener tamao suficiente para
   que quepa el sprite completo (debe de ser de Ancho*Alto elementos).    
                                                                          
   Es el proceso contrario de dibujar un sprite: se trata de leerlo desde 
   la pantalla (tanto fsica como virtual) a un array.                    
  
*/
void GetSprite( int x, int y, int ancho, int alto, char far *bitmap, 
		 unsigned int segmento )
{
unsigned int offs;

 offs = (y*320)+x;

asm {
	push ds                               /* preservar el valor de DS */
	les di, bitmap                        /* ES:DI = Bitmap[0] */
	mov ds, [segmento]
	mov si, [offs]                        /* DS:SI = (x,y) */
	mov dx, [alto]                        /* DX = Contador bucle */
	mov bx, [ancho]                       /* para mayor velocidad */
     }
BucleSpriteYR:;
asm {
	mov cx, bx                            /* CX = Ancho */
	rep movsb                             /* movemos CX bytes */
	add si, 320                           /* siguiente lnea de pant. */
	sub si, bx
	dec dx                                /* hasta que DX = 0 */
	jnz BucleSpriteYR

	pop ds                                /* recuperamos DS */
    }
}

/*--------------------------------------------------------------------------
 [END OF FILE]                                                   [NoP]
 --------------------------------------------------------------------------*/
