/*                                                                          
  Ϳ
   VScreen.h                                                              
                                                                          
   Librera para peticin de memoria para Pantallas Virtuales para el     
   modo 13h (320x200x256). Contiene tambin rutinas de volcado y borrado. 
                                                                          
   Para incluirla en cualquier programa C basta con usar la orden         
   #include "vscreen.h"                   al principio del mismo.         
                                                                          
   Cdigo y texto:  [NoP] de Compiler SoftWare.                     
  1997 C.S.
*/

/*  Funciones implementadas:  */

unsigned int AllocateVScreen( void );                    /* pedir memoria */
char FreeVScreen ( unsigned int );                       /* liberar ""    */
void FlipVScreen ( unsigned int, unsigned int );         /* volcado de VS */
void ClearVScreen ( unsigned int, unsigned char );       /* borrado de VS */


/*                                                                          
  Ŀ
   segm = AllocateVScreen( void );                                        
                                                                          
   Pide al DOS un segmento de memoria para utilizarlo como pantalla vir-  
   tual (65536 bytes) y devuelve el segmento liberado o 0 en caso de que  
   haya ocurrido algn error.                                             
  
*/
unsigned int AllocateVScreen( void )
{

 unsigned int segment, error;
 error = 0;

 asm {
	mov ah, 0x48                                /* peticin de memoria */
	mov bx, 4096                                /* 4096*16 = 65536 */
	int 0x21                                    /* interrupcin 21h */
	jc ErrorAVSC
	mov [segment], ax
	jmp ExitAVSC                                /* control de errores */
    }

ErrorAVSC:;
   asm mov [error], bx

ExitAVSC:;
   if(error !=0 ) return( 0 );
   return(segment);                                 /* devolucin de segm */
   
}

/*                                                                          
  Ŀ
   retc = FreeVScreen( segm );                                            
                                                                          
   Libera un segmento de memoria previamente liberado. Devuelve 0 en caso 
   de error o 1 si ha tenido xito.                                       
  
*/
char FreeVScreen( unsigned int segment ) 
{

asm {
	mov es, [segment]                           /* ES = Segmento a lib.*/
	mov ah, 0x49                                /* liberacin de mem. */
	int 0x21                                    /* interrupcin 21h */
	jc ErrorFVSC
	jmp ExitFVSC                                /* control de errores */
    }
  
ErrorFVSC:;
   return(0);

ExitFVSC:; 
   return(1);                                       /* devolver cdigo err.*/
}

/*                                                                          
  Ŀ
   FlipVScreen( origen, destino );                                        
                                                                          
   Vuelca un segmento (origen) sobre otro (destino).                      
  
*/
void FlipVScreen( unsigned int source, unsigned int dest )
{ 
  
asm {
	push ds
	mov es, [dest]
	mov ds, [source]
	xor si, si                  
	xor di, di                  
	mov cx, 32000
	rep movsw                   
	pop ds
    }

}

/*                                                                          
  Ŀ
   ClearVScreen( segmento, valor );                                       
                                                                          
   Rellena un segmento de memoria con el valor especificado.              
  
*/
void ClearSegment ( unsigned int dest, unsigned char valor )
{ 
  
asm {
	mov es, [dest]          
	xor di, di                  
	mov al, [valor] 
	mov ah, al
	mov cx, 32000
	rep stosb                   
    }
}

